import functools
import warnings


def deprecated(deadline_date, msg=''):
    def deprecated_decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            warnings.warn(
                '{} {} is deprecated API and will be removed on {}. {}'.format(
                    args[0].__class__.__name__, func.__name__,
                    deadline_date, msg).strip(),
                FutureWarning
            )
            return func(*args, **kwargs)
        return wrapper
    return deprecated_decorator



class NotificationTransport(object):
    WEBHOOK = 0
    WEBSOCKETS_CLIENT = 1
    WEBSOCKETS_SERVER = 2


class NotificationHTTPMethod(object):
    POST = 0
    GET = 1


class Conf(object):
    NEW = 'new'
    EXACT = 'exact'
    HA = 'ha'
    JUNK = 'junk'
    NM = 'nm'
    DET = 'det'
    REINIT = 'reinit'


class Licence(object):
    BASIC = 'basic'
    STANDARD = 'standard'
    STANDARD_PLUS = 'standard+'
    ADVANCED = 'advanced'



class PersonsLists:

    def __init__(self, http_client):
        self.http_client = http_client

    def create(self, name):
        data = {'name': name}
        return self.http_client.post('v1/lists/persons/', data=data)

    def list(self, **kwargs):
        return self.http_client.get('v1/lists/persons/', params=kwargs)

    def get(self, pl_id):
        return self.http_client.get('v1/lists/persons/{}/'.format(pl_id))

    def update(self, pl_id, name):
        data = {'name': name}
        return self.http_client.patch('v1/lists/persons/{}/'.format(pl_id), data=data)

    def delete(self, pl_id):
        return self.http_client.delete('v1/lists/persons/{}/'.format(pl_id))

    def idxids_list(self, pl_id):
        return self.http_client.get('v1/lists/persons/{}/idxids/'.format(pl_id))

    def create_idxids(self, pl_id, idxids):
        data = {'idxids': idxids}
        return self.http_client.post('v1/lists/persons/{}/idxids/'.format(pl_id), data=data)

    def delete_idxids(self, pl_id, idxids):
        data = {'idxids': idxids}
        return self.http_client.delete('v1/lists/persons/{}/idxids/'.format(pl_id), data=data)

    def exended_idxids_list(self, pl_id, idxids):
        data = {'idxids': idxids}
        return self.http_client.get('v1/lists/persons/{}/idxids/extended/'.format(pl_id), data=data)

    def delete_idxids_in_lists(self, list_ids, idxids):
        data = {
            'list_ids': list_ids,
            'idxids': idxids
        }
        return self.http_client.delete('v1/lists/persons/idxids/', data=data)

    def create_idxids_in_lists(self, list_ids, idxids):
        data = {
            'list_ids': list_ids,
            'idxids': idxids
        }
        return self.http_client.post('v1/lists/persons/idxids/', data=data)

    def delete_all_idxids(self, idxid):
        return self.http_client.delete('v1/lists/persons/{}/idxids/all/'.format(idxid))

    def delete_idxid_from_lists(self, list_ids, idxid):
        data = {'lists_ids': list_ids}
        return self.http_client.post('v1/lists/persons/idxids/{}/'.format(idxid), data=data)

    def delete_idxid_from_all_lists(self, idxid):
        return self.http_client.post('v1/lists/persons/idxids/{}/all/'.format(idxid))



class Entries(object):

    def __init__(self, http_client):
        self.http_client = http_client

    def list(self, **kwargs):
        return self.http_client.get('v1/entries/', params=kwargs)

    def delete(self, id):
        return self.http_client.delete('v1/entries/{}/'.format(id))

    def stats_idxid(self, idxid):
        return self.http_client.get('v1/entries/stats/idxid/{}/'.format(idxid))

    def stats_sources(self, **kwargs):
        return self.http_client.get('v1/entries/stats/sources/', params=kwargs)
